package com.javacodegeeks.basics_of_servlets;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Objects;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// only responds to POST requests
@WebServlet("/jsonbodyservlet")
public final class JsonBodyServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {

        final String requestBody = readBody(req);
        echoToClient(resp, requestBody);
    }

    private String readBody(final HttpServletRequest req) throws IOException {
        assert !Objects.isNull(req);

        final StringBuilder data = new StringBuilder();
        
        // get a reader to read the body contents
        try (BufferedReader reader = req.getReader()) {
            String contents = null;
            while ((contents = reader.readLine()) != null) {
                data.append(contents);
            }
        }

        return data.toString();
    }

    private void echoToClient(final HttpServletResponse resp, final String data) throws IOException {
        assert !Objects.isNull(resp);
        assert !Objects.isNull(data);

        // echo the body contents back to the client
        try (PrintWriter writer = resp.getWriter()) {
            resp.setContentType("text/plain");

            writer.write(data);
            writer.flush();
        }
    }
}
