package com.javacodegeeks.basics_of_servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Objects;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// handles both GET and POST requests
@WebServlet("/enumerator")
public final class RequestEnumerator extends HttpServlet {

    private static final long serialVersionUID = 1L;

    @Override
    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
        doRequest(req, resp); // enumerate the request parameters
    }

    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
        doRequest(req, resp); // enumerate the request parameters
    }

    private void doRequest(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
        assert !Objects.isNull(req) && !Objects.isNull(resp);

        // Get the write and specify the content type, the write to the response
        // stream
        try (PrintWriter writer = resp.getWriter()) {
            resp.setContentType("text/plain");

            enumerateRequestParamters(writer, req);

            writer.flush();
        }
    }

    private void enumerateRequestParamters(final PrintWriter writer, final HttpServletRequest request) {
        assert !Objects.isNull(writer);
        assert !Objects.isNull(request);

        // this will return an enumeration of all request parameters in query
        // string eg: ?name=joe&age=30
        final Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            final String name = parameterNames.nextElement();
            writer.write(name);
            writer.write(":\t");

            // important to note is that a parameter name can occur more than
            // once hence we iterate
            for (String val : request.getParameterValues(name)) {
                writer.write(val);
                writer.write("\t");
            }

            writer.write("\n");
        }
    }
}
