package com.javacodegeeks.basics_of_servlets.filters;

import java.io.IOException;
import java.util.Objects;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.annotation.WebInitParam;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// valid for the enumerator route
// we also specify a header to look out for to block requests if the header is present
@WebFilter(urlPatterns = "/enumerator", initParams = { @WebInitParam(name = BlockingFilter.DISALLOW_HEADER_KEY, value = "poisonpill") })
public class BlockingFilter implements Filter {
    
    static final String DISALLOW_HEADER_KEY = "disallow-key";
    
    private String disallowHeaderValue;

    @Override
    public void init(final FilterConfig filterConfig) throws ServletException {
        // get the header value
        this.disallowHeaderValue = filterConfig.getInitParameter(DISALLOW_HEADER_KEY);
    }

    @Override
    public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException {
        final String value = ((HttpServletRequest) request).getHeader(this.disallowHeaderValue);
        final boolean disallow = Objects.isNull(value) ? false : true;
        
        // continue the request via the filter pipeline if the header is not present
        if (!disallow) {
            chain.doFilter(request, response);
        } else {
            
            // do not continue the filter pipeline but respond back to client
            final HttpServletResponse resp = (HttpServletResponse) response;
            resp.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            resp.setContentType("text/plain");            
            resp.getWriter().write("Poison pill detected, stopping request");
        }
    }

    @Override
    public void destroy() {
    }
}
